/* externs.h */
/* included by every file; sets up our system declarations */

#define _GNU_SOURCE	// Use the GNU C Library Extensions

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

/* GCC gives warnings for non-pointers set to NULL */
#undef NULL
#define NULL 0

/* Most C libraries map ctype.h functions into memory, which can slow things
   down quite a bit. Here we use our own macros and avoid locale-based maps. */
   
#undef isspace
#undef isdigit
#undef isalpha
#undef isalnum
#undef isxdigit
#define isspace(c)  ({ char _c=c; (_c == ' ' || _c == '\t'); })
#define isdigit(c)  ({ char _c=c; (_c >= '0' && _c <= '9'); })
#define isalpha(c)  ({ char _c=(c) | 32; (_c >= 'a' && _c <= 'z'); })
#define isalnum(c)  ({ char _ch=c; (isdigit(_ch) || isalpha(_ch)); })
#define isxdigit(c) ({ char _ch=c; (isdigit(_ch) || \
		       ((_ch | 32) >= 'a' && ((_ch | 32) <= 'f'))); })

#undef toupper
#undef tolower
#define toupper(c)  ({ char _c=c; (_c < 'a' || _c > 'z')?_c:(_c & ~32); })
#define tolower(c)  ({ char _c=c; (_c < 'A' || _c > 'Z')?_c:(_c | 32); })

/* Like strncpy but doesn't pad with zeroes. n=buffer size instead of siz-1 */
#define strmaxcpy(d, s, n) ({ char *_d=d; const char *_s=s; \
                              int _n=n, _len=strlen(_s)+1; \
                              if(_n > 0) { \
                                memcpy(_d, _s, (_len > _n)?_n:_len); \
                                _d[_n-1]='\0'; \
                              } _d; })


/* Global Definitions */

extern int width, height, fontwidth, fontheight, refresh;
extern int cursor_start, cursor_end, underline, border_color;
extern int blink, isofont, lineart, palette;
extern int maxclk, options, cardnum, chipset, clockchip;

extern char *modeline, *fontprog, *resetprog;

extern int verbose, force;

extern char *modeline_flags[];

/* Linked-list of modelines read in from configuration files */
extern struct modeline {
  struct modeline *next;
  char *name;
  int clock;
  int timing[8];
  int width;
  int height;
  int flags;
} *modelist;

/* Linked-list of font sizes associated with a specific font name */
extern struct font {
  struct font *next;
  char *name;
  int width;
  int height;
} *fontlist;

extern struct range {
  struct range *next;
  int lo;
  int hi;
} *horiz_ranges, *vert_ranges;

/* Modeline Flags */

#define NHSYNC     0x1
#define NVSYNC     0x2
#define DOUBLESCAN 0x4
#define INTERLACE  0x8
#define CSYNC      0x10

/* Chipset Options */

#define LOADFONT	0x1
#define CLOCKDIV2	0x2

#define SLOW_DRAM	0x10
#define MED_DRAM	0x20
#define FAST_DRAM	0x40
#define XFAST_DRAM	0x80
#define HIBIT_LOW	0x100
#define HIBIT_HIGH	0x200
#define SWAP_HIBIT	0x400
#define S3_HSTEXT	0x800
#define LEGEND		0x1000
#define ET4000_ALTCLK	0x2000
#define SPEA_MERCURY	0x4000
#define CLOCKCHIP_X	0x8000
#define TOPLEFT		0x10000
#define STRETCH		0x20000


/* External Function Declarations */

/* config.c */
extern int read_config(char *filename);

/* ttyresize.c */
extern int resize_tty(int cols, int rows);
extern int load_bios_font(int fontsize);

