/* pci.h */

#define NUM_CARDS 16

extern struct pcicard {
  unsigned int device;
  unsigned int vendor;
  int address;
  int fb;
  int ioaddr;

  int initialized;
  int chipset;
  int memory;
  int maxclock;
} card_list[NUM_CARDS];

extern int pci_enable_device(int device);
extern int pci_find_vendor_vga(int verbose);


/* PCI Read/Write Macros */

extern inline
unsigned char pci_read_config_byte(int device, int where) {
  outl(0x80000000 | ((device & 0xffff) << 8) | where, 0xCF8);
  return inb(0xCFC | (where&3));
}

extern inline
void pci_write_config_byte(int device, int where, unsigned char value) {
  outl(0x80000000 | ((device & 0xffff) << 8) | where, 0xCF8);
  outb(value, 0xCFC | (where&3));
}

extern inline
short pci_read_config_word(int device, int where) {
  outl(0x80000000 | ((device & 0xffff) << 8) | where, 0xCF8);
  return inw(0xCFC | (where&2));
}

extern inline
void pci_write_config_word(int device, int where, short value) {
  outl(0x80000000 | ((device & 0xffff) << 8) | where, 0xCF8);
  outw(value, 0xCFC | (where&2));
}

extern inline
int pci_read_config_dword(int device, int where) {
  outl(0x80000000 | ((device & 0xffff) << 8) | where, 0xCF8);
  return inl(0xCFC);
}

extern inline
void pci_write_config_dword(int device, int where, int value) {
  outl(0x80000000 | ((device & 0xffff) << 8) | where, 0xCF8);
  outl(value, 0xCFC);
}
