/*
 * Copyright 1999 Egbert Eich
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the authors not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The authors makes no representations
 * about the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * THE AUTHORS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sys/io.h>

#define DEBUG

typedef unsigned char CARD8;
typedef unsigned short CARD16;
typedef unsigned int CARD32;

#if defined (__alpha__) || defined (__ia64__)
typedef unsigned long memType;
#else
typedef unsigned int memType;
#endif

struct regs86 {
  long ebx, ecx, edx, esi, edi, ebp, eax, eip, esp;
  unsigned short cs, ss, es, ds, fs, gs;
  long eflags;
};

typedef struct {
  CARD32 ax, bx, cx, dx, cs, es, ds, si, di;
} i86biosRegs, *i86biosRegsPtr;

enum dev_type { NONE, ISA, PCI };

extern struct device {
  enum dev_type type;
  union {
    int none;
    struct pci {
      int bus;
      int dev;
      int func;
    } pci;
  } loc;
} Device;

#ifdef __alpha__
unsigned long _bus_base(void);
extern void *vram_map;
extern int sparse_shift;
#endif

extern struct config Config;
#define IOPERM_BITS 1024
extern int ioperm_list[IOPERM_BITS];

extern void setup_io(void);
extern void do_x86(unsigned long bios_start,i86biosRegsPtr regs);
extern int run_bios_int(int num, struct regs86 *regs);
extern CARD32 getIntVect(int num);
CARD32 getIP(void);

extern void call_boot(struct device *dev);
extern void runINT(int num,i86biosRegsPtr Regs);
extern void add_hlt(unsigned long addr);
extern void del_hlt(int addr);
extern void list_hlt();

#ifdef __alpha__
CARD8 mem_rb(CARD32 addr);
CARD16 mem_rw(CARD32 addr);
CARD32 mem_rl(CARD32 addr);
void mem_wb(CARD32 addr, CARD8 val);
void mem_ww(CARD32 addr, CARD16 val);
void mem_wl(CARD32 addr, CARD32 val);
#endif

extern int int_handler(int num, struct regs86 *regs);

#define MEM_FILE "/dev/mem"

#ifndef V_BIOS
#define V_BIOS 0xc0000
#endif

#ifdef DEBUG
# define debug(fmt,a...) fprintf(stderr, fmt, ##a)
#else
# define debug(fmt,a...)
#endif

#include "pci.h"
